//
//  HttpsUtil.m
//  HttpsCertificationDemo
//
//  @author wlf(Andy)
//  @datetime 2016-02-26 12:02 GMT+8
//  @email 411086563@qq.com
//

#import "HttpsUtil.h"

@implementation HttpsUtil

// 双向认证
+ (void)configHTTPSessionManager:(AFHTTPSessionManager *) manager serverCers:(NSArray *) serverCerNames clientP12:(NSString *) clientp12Name clientP12Password:(NSString *) clientP12Password isSelfCa:(BOOL) isSelfCa{
    
    __weak AFHTTPSessionManager*_manager = manager;
    
    if(!_manager){
        return;
    }
    
    NSMutableSet * serverCerDatas= [[NSMutableSet alloc] init];
    
    for (NSString * serverCerName in serverCerNames){
        if(!serverCerName){
            continue;
        }
        NSString *cerPath = [[NSBundle mainBundle] pathForResource:serverCerName ofType:nil];// 证书的路径
        NSData *certData = [NSData dataWithContentsOfFile:cerPath];
        [serverCerDatas addObject:certData];
    };
    
    _manager.securityPolicy.pinnedCertificates=serverCerDatas;
    
    // 需要验证客户端证书（双向认证）
    if(clientp12Name && clientP12Password){
        
        [manager setSessionDidReceiveAuthenticationChallengeBlock:^NSURLSessionAuthChallengeDisposition(NSURLSession*session, NSURLAuthenticationChallenge *challenge, NSURLCredential *__autoreleasing*_credential) {
            
            NSURLSessionAuthChallengeDisposition disposition = NSURLSessionAuthChallengePerformDefaultHandling;
            
            __autoreleasing NSURLCredential *credential =nil;
            
            if([challenge.protectionSpace.authenticationMethod isEqualToString:NSURLAuthenticationMethodServerTrust]) {
                
                if([_manager.securityPolicy evaluateServerTrust:challenge.protectionSpace.serverTrust forDomain:challenge.protectionSpace.host]){
                    
                    credential = [NSURLCredential credentialForTrust:challenge.protectionSpace.serverTrust];
                    
                    if(credential) {
                        disposition =NSURLSessionAuthChallengeUseCredential;
                    } else {
                        disposition =NSURLSessionAuthChallengePerformDefaultHandling;
                    }
                } else {
                    disposition = NSURLSessionAuthChallengeCancelAuthenticationChallenge;
                }
            } else {
                // client authentication
                SecIdentityRef identity = NULL;
                SecTrustRef trust = NULL;
                NSString *p12Path = [[NSBundle mainBundle] pathForResource:clientp12Name ofType:nil];
                NSFileManager *fileManager =[NSFileManager defaultManager];
                
                if(![fileManager fileExistsAtPath:p12Path]){
                    NSLog(@"客户端证书不存在！");
                } else {
                    NSData *PKCS12Data = [NSData dataWithContentsOfFile:p12Path];
                    
                    if ([[self class]extractIdentity:&identity andTrust:&trust fromPKCS12Data:PKCS12Data pkcs12Password:clientP12Password]) {
                        NSLog(@"加载客户端证书成功");
                        SecCertificateRef certificate = NULL;
                        SecIdentityCopyCertificate(identity, &certificate);
                        const void*certs[] = {certificate};
                        CFArrayRef certArray =CFArrayCreate(kCFAllocatorDefault, certs,1,NULL);
                        credential =[NSURLCredential credentialWithIdentity:identity certificates:(__bridge  NSArray*)certArray persistence:NSURLCredentialPersistencePermanent];
                        disposition =NSURLSessionAuthChallengeUseCredential;
                    }
                }
            }
            *_credential = credential;
            return disposition;
        }];
    }
    
    if(isSelfCa){
        manager.securityPolicy.allowInvalidCertificates =YES;
        manager.securityPolicy.validatesDomainName=NO;
        // 以上是针对自签名CA颁发的server证书必须设置的，除了之外还需要在Info.plist加上
        
        /*
         <key>NSAppTransportSecurity</key>
         <dict>
         <!--
         <key>NSAllowsArbitraryLoads</key>
         <true/>
         -->
         <key>NSExceptionDomains</key>
         <dict>
         <key>www.andy5.me</key>
         <dict>
         <key>NSIncludesSubdomains</key>
         <true/>
         <key>NSTemporaryExceptionAllowsInsecureHTTPLoads</key>
         <true/>
         <!--
         <key>NSTemporaryExceptionMinimumTLSVersion</key>
         <string>TLSv1.0</string>
         <key>NSExceptionRequiresForwardSecrecy</key>
         <false/>
         -->
         </dict>
         </dict>
         </dict>
         */
    }else{
        // 注释掉Info.plist中整个NSAppTransportSecurity节点的配置
    }
}

// 单向认证
+ (void)cconfigHTTPSessionManager:(AFHTTPSessionManager *)manager serverCers:(NSArray *) serverCerNames isSelfCa:(BOOL) isSelfCa{
    
    [[self class]configHTTPSessionManager:manager serverCers:serverCerNames clientP12:nil clientP12Password:nil isSelfCa:isSelfCa];
}

// 解压初始化客户端证书
+ (BOOL)extractIdentity:(SecIdentityRef*)outIdentity andTrust:(SecTrustRef *)outTrust fromPKCS12Data:(NSData *)inPKCS12Data pkcs12Password:(NSString *) p12Password{
    
    OSStatus securityError = errSecSuccess;
    
    NSDictionary* optionsDictionary = [NSDictionary dictionaryWithObject:p12Password forKey:(__bridge id)kSecImportExportPassphrase];
    
    CFArrayRef items = CFArrayCreate(NULL, 0, 0, NULL);
    securityError = SecPKCS12Import((__bridge CFDataRef)inPKCS12Data,(__bridge CFDictionaryRef)optionsDictionary,&items);
    
    if(securityError == 0) {
        // 成功
        CFDictionaryRef myIdentityAndTrust =CFArrayGetValueAtIndex(items,0);
        const void*tempIdentity =NULL;
        tempIdentity= CFDictionaryGetValue (myIdentityAndTrust,kSecImportItemIdentity);
        *outIdentity = (SecIdentityRef)tempIdentity;
        const void*tempTrust =NULL;
        tempTrust = CFDictionaryGetValue(myIdentityAndTrust,kSecImportItemTrust);
        *outTrust = (SecTrustRef)tempTrust;
    } else {
        NSLog(@"初始化客户端证书失败，errorCode：%d",(int)securityError);
        return NO;
    }
    return YES;
}

@end
