package org.wlf.https_certification;

import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.view.View;
import android.webkit.WebView;
import android.widget.TextView;
import android.widget.Toast;

import org.wlf.https_certification.util.HttpsUtil;

import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.TimeUnit;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocketFactory;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.internal.tls.OkHostnameVerifier;

public class MainActivity extends AppCompatActivity {

    private TextView mTvResult;
    private WebView mWvResult;

    private OkHttpClient mOkHttpClient;

    private static final String SERVER_URL = "www.andy5.me";
    private static final String HTTPS_SERVER_URL = "https://" + SERVER_URL;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_main);
        mTvResult = (TextView) findViewById(R.id.tvResult);
        mWvResult = (WebView) findViewById(R.id.wvResult);

        initHttpsEngine(true);// 使用自签名CA给服务器server证书签名的使用true，第三方权威CA签名的使用false
    }

    private void initHttpsEngine(boolean isSelfCa) {
        try {
            // 初始化服务器端公钥证书，得到SSLSocketFactory
            SSLSocketFactory sslSocketFactory = HttpsUtil.getSslSocketFactory(new InputStream[]{getAssets().open
                    ("server.cer")}, getAssets().open("client_cert_lib.bks"), "c123456");
            OkHttpClient.Builder builder = new OkHttpClient.Builder();

            if (isSelfCa) {
                /**
                 * 注意；如果你的server.cer是来自自签名CA颁发的，那么就要设置下面的customVerifier，主要是为了解决报以下异常，
                 * 即跳过Hostname www.andy5.me在CA上的验证，如果你的server.cer是来自第三方SSL权威机构颁发的，不用设置这个customVerifier
                 *
                 * javax.net.ssl.SSLPeerUnverifiedException: Hostname www.andy5.me not verified:
                 * certificate: sha1/EnrjjhNxjvuDkO/rJqPmJ9XaIMs=
                 * DN: CN=Andy Wu(www.andy5.me),OU=Andy5 Server,O=www.andy5.me,L=Guangzhou,ST=Guangdong,C=CN
                 * subjectAltNames: []
                 */
                HostnameVerifier customVerifier = new HostnameVerifier() {
                    @Override
                    public boolean verify(String hostname, SSLSession session) {
                        // 指定SERVER_URL一定可以通过
                        if (SERVER_URL.equalsIgnoreCase(hostname)) {
                            return true;
                        } else {
                            // 使用默认的OkHostnameVerifier进行验证
                            return OkHostnameVerifier.INSTANCE.verify(hostname, session);
                        }
                    }
                };
                mOkHttpClient = builder.sslSocketFactory(sslSocketFactory).connectTimeout(30, TimeUnit.SECONDS)
                        .hostnameVerifier(customVerifier).build();
            } else {
                mOkHttpClient = builder.sslSocketFactory(sslSocketFactory).connectTimeout(30, TimeUnit.SECONDS).build();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void testHttps(View v) {

        if (mOkHttpClient == null) {
            Toast.makeText(getApplicationContext(), "请先初始化客户端密钥库和服务器端公钥！", Toast.LENGTH_SHORT).show();
            return;
        }

        mTvResult.setText("正在从 " + HTTPS_SERVER_URL + " 获取数据....");
        mWvResult.loadData("", "text/html", "UTF-8");

        Request request = new Request.Builder().url(HTTPS_SERVER_URL).build();
        Call call = mOkHttpClient.newCall(request);
        call.enqueue(new Callback() {
            @Override
            public void onFailure(Call call, final IOException e) {
                e.printStackTrace();
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        mTvResult.setText("从 " + HTTPS_SERVER_URL + " 获取数据失败！\n" + e);
                    }
                });
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                final String html = response.body().string();
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        mTvResult.setText("从 " + HTTPS_SERVER_URL + " 获取数据成功！");
                        mWvResult.loadData(html, "text/html", "UTF-8");
                    }
                });
            }
        });
    }
}
